# Copyright (c) 2014 Dave Love, Liverpool University
# The licence for this file is as for the package itself.

# soname version
%global sover 1.0

# tarball unpacks to mrmpi-%orig_version
# don't use this for package version as it won't order correctly
%global orig_version 7Apr14

%if 0%{?el6}
%global mpich mpich2
%global mpich_load %_mpich2_load
%global mpich_unload %_mpich2_unload
%else
%global mpich mpich
%global mpich_load %_mpich_load
%global mpich_unload %_mpich_unload
%endif

Name:           mrmpi
Version:        20140714
Release:        2%{?dist}
Summary:        MapReduce-MPI

License:        BSD
URL:            http://mapreduce.sandia.gov/
Source0:        http://www.cs.sandia.gov/~sjplimp/tars/mrmpi.tar.gz

BuildRequires:  gcc-c++
BuildRoot:      %{_tmppath}/%name-%version-%{release}-root-%(%{__id_u} -n)
Requires: %name-common = %version-%release

%global desc \
The MapReduce-MPI (MR-MPI) library implements the MapReduce operation\
popularized by Google on top of standard MPI message passing.\
\
The library is designed for parallel execution on distributed-memory\
platforms, but will also operate on a single processor. It requires no\
additional software to build and run, except linking with an MPI\
library if you wish to perform MapReduces in parallel. Similar to the\
original Google design, a user performs a MapReduce by writing a small\
program that invokes the library. The user typically provides two\
application-specific functions, a "map()" and a "reduce()", that are\
called back from the library when a MapReduce operation is executed.\
"Map()" and "reduce()" are serial functions, meaning they are invoked\
independently on individual processors on portions of your data when\
performing a MapReduce operation in parallel.

%description
%desc

This is a serial version.

%package openmpi
Summary: MapReduce-MPI
BuildRequires: openmpi-devel gcc-c++
Requires: %name-common = %version-%release, openmpi

%description openmpi
%desc

This is a parallel version using openmpi.

%package %mpich
Summary: MapReduce-MPI
BuildRequires: %mpich-devel gcc-c++
Requires: %name-common = %version-%release, %mpich

%description %mpich
%desc

This is a parallel version using %mpich.

%package common
Summary: MapReduce-MPI
BuildRequires: python-devel
BuildArch:	noarch

%description common
%desc

This package contains files common to the serial and parallel versions.

%package        devel
Summary:        Development files for %{name}
Group:          Development/Libraries
Requires:	%{name} = %{version}-%{release}

%description    devel
This package contains libraries and header files for
developing applications that use %{name}.

%package        openmpi-devel
Summary:        Development files for %{name}-openmpi
Group:          Development/Libraries
Requires:       %{name}-openmpi = %{version}-%{release}

%description    openmpi-devel
The %{name}-openmpi-devel package contains libraries and header files for
developing applications that use %{name}-openmpi.

%package        %mpich-devel
Summary:        Development files for %{name}-%mpich
Group:          Development/Libraries
Requires:       %{name}-%mpich = %{version}-%{release}

%description    %mpich-devel
The %{name}-%mpich-devel package contains libraries and header files for
developing applications that use %{name}-%{mpich}.

%package        doc
Summary:        Documentation for %{name}
BuildArch:	noarch

%description    doc
Documentation for %{name}

%prep
%setup -q -n mrmpi-%orig_version
chmod 644 examples/in.sssp

%build

make -C mpistubs CXXFLAGS=-fPIC CC='g++ -g'
ln -s libmpi_stubs.a mpistubs/libmpi.a
make -C src %{?_smp_mflags} -f Makefile.shlib serial CC='g++ -g' SHLIBFLAGS='-shared -Wl,-soname,lib%name.so.%sover -L../../mpistubs'
make -C src %{?_smp_mflags} serial CC='g++ -g'
make -C oink %{?_smp_mflags} serial
mv -f src/libmrmpi_serial.so src/lib%{name}.so.%sover
rm src/libmrmpi_serial.a

%global dobuild() \
rm -rf src/Obj_*mpicc; \
make -C src %{?_smp_mflags} -f Makefile.shlib mpicc SHLIBFLAGS='-shared -Wl,-soname,%name$MPI_SUFFIX.so.%sover'; \
make -C src %{?_smp_mflags} mpicc; \
rm -rf oink/Obj_mpicc; \
make -C oink %{?_smp_mflags} mpicc; \
mv oink/oink_mpicc oink/oink$MPI_SUFFIX; \
mv -f src/libmrmpi_mpicc.so src/lib%{name}$MPI_SUFFIX.so.%sover; \
rm src/libmrmpi_mpicc.a

%_openmpi_load
%dobuild
%_openmpi_unload

%mpich_load
%dobuild
%mpich_unload

%install
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT%python_sitelib
mkdir -p $RPM_BUILD_ROOT%_includedir/%name $RPM_BUILD_ROOT%_libdir
pushd python
python install.py $RPM_BUILD_ROOT%_libdir $RPM_BUILD_ROOT%python_sitelib
popd
# fixme: are they all relevant?
install -m644 src/*.h $RPM_BUILD_ROOT%_includedir/%{name}
install src/lib%{name}.so.%sover $RPM_BUILD_ROOT%_libdir
ln -s lib%{name}.so.%sover $RPM_BUILD_ROOT%_libdir/lib%{name}.so
install -D oink/oink_serial $RPM_BUILD_ROOT%_bindir/oink
%global inst \
mkdir -p $RPM_BUILD_ROOT$MPI_INCLUDE/%name $RPM_BUILD_ROOT$MPI_BIN; \
install -D src/lib%{name}$MPI_SUFFIX.so.%sover $RPM_BUILD_ROOT$MPI_LIB/lib%{name}.so.%sover; \
ln -s lib%{name}.so.%sover $RPM_BUILD_ROOT$MPI_LIB/lib%{name}.so; \
install oink/oink$MPI_SUFFIX $RPM_BUILD_ROOT$MPI_BIN; \
install -m644 src/*.h $RPM_BUILD_ROOT$MPI_INCLUDE/%name
%_openmpi_load
%inst
%_openmpi_unload
%mpich_load
%inst
%mpich_unload
cp -a oinkdoc doc/oink
rm -f doc/oink/Manual.py
for f in examples/*.py; do sed -i -e 's,/usr/local,/usr,' $f; done
cp -a examples doc
chmod -x doc/oink/*

%post -p /sbin/ldconfig
%postun -p /sbin/ldconfig

%clean
rm -rf $RPM_BUILD_ROOT

%files
%defattr(-,root,root,-)
%_libdir/*.so.*
%_bindir/*

%files openmpi
%defattr(-,root,root,-)
%_libdir/openmpi/lib/*.so.*
%_libdir/openmpi/bin/*

%files %mpich
%defattr(-,root,root,-)
%_libdir/%mpich/lib/*.so.*
%_libdir/%mpich/bin/*

%files common
%defattr(-,root,root,-)
%doc README LICENSE
%python_sitelib/*

%files openmpi-devel
%defattr(-,root,root,-)
%_includedir/openmpi-%{_arch}/%{name}
%_libdir/openmpi/lib/*.so

%files %mpich-devel
%defattr(-,root,root,-)
%_includedir/%mpich-%{_arch}/%{name}
%_libdir/%mpich/lib/*.so

%files devel
%defattr(-,root,root,-)
%_includedir/%{name}
%_libdir/*.so

%files doc
%defattr(-,root,root,-)
%doc README LICENSE doc/*

%changelog
* Mon Jul 21 2014 Dave Love <d.love@liverpool.ac.uk> - 20140714-2
- openmpi package depends on openmpi, not mpich

* Sat Jun  7 2014 Dave Love <d.love@liverpool.ac.uk> - 20140714-1
- New version
- Build mpich version and doc and common packages

* Tue Feb 18 2014 Dave Love <d.love@liverpool.ac.uk> - 22Nov13-1
- Initial packaging
